// ****************************************************************
//
//   IBM Confidential
//   OCO Source Materials
//   <<PART NUMBER - 5724-B75>>
//   (C) Copyright IBM Corp. 2001 - All Rights Reserved.
//   US Government Users Restricted Rights - Use, duplication or disclosure
//   restricted by GSA ADP Schedule Contract with IBM Corp.
//
// ****************************************************************

package com.ibm.etools.cobol.importer;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.StreamTokenizer;
import java.io.StringReader;
import java.io.StringWriter;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Stack;
import java.util.StringTokenizer;
import java.util.Vector;

public class CobolCommareaGenerator extends StringWriter
{
	private static String copyright = "(c) Copyright IBM Corporation 1998, 1999.";
	private BufferedReader f_in = null;
	private String inFile = new String();
	private static int commareaLevel = 1;
	
public CobolCommareaGenerator(org.eclipse.emf.ecore.resource.URIConverter converter, String inputFile) throws CobolException
{
	inFile = inputFile;
	try
	{
//		f_in = new BufferedReader(new FileReader(inputFile));

		try {
			URL url = new URL(inputFile);
			inputFile = url.getPath();
		} catch (MalformedURLException e) {
			// does nothing if is not an URL
		}
		
		File file = new File(inputFile);
		 
		org.eclipse.emf.common.util.URI inputURI = null;
		
		if (file.exists())
			inputURI = org.eclipse.emf.common.util.URI.createFileURI(inputFile);
		else
			inputURI = org.eclipse.emf.common.util.URI.createURI(inputFile);
			
		f_in = new BufferedReader(new InputStreamReader(converter.createInputStream(inputURI)));
		
	} catch (Exception exc) {
		//DEFECT 22348
		try {
			f_in = new BufferedReader(new FileReader(inFile));
		} catch (FileNotFoundException e) {
			Object[] obj = {inFile};
			throw new CobolException(CobolResource.getString("_EXC_Error_opening_file", obj));
		}
	}	
}

public CobolCommareaGenerator(String inputFile) throws CobolException
{
	this(new org.eclipse.emf.ecore.resource.impl.URIConverterImpl(), inputFile);
}

public CobolCommareaGenerator(org.eclipse.emf.ecore.resource.URIConverter converter, org.eclipse.emf.common.util.URI inputURI) throws CobolException
{
	inFile = inputURI.toString();
	
	try
	{
		f_in = new BufferedReader(new InputStreamReader(converter.createInputStream(inputURI)));
		
	} catch (Exception exc) {
		Object[] obj = {inFile};
		throw new CobolException(CobolResource.getString("_EXC_Error_opening_file", obj));
	}	
}

public void closeFile() throws CobolException
{
	try {
		
		f_in.close();
		
	} catch (IOException exc) {
		Object[] obj = {inFile};
		throw new CobolException(CobolResource.getString("_EXC_Error_closing_file", obj));			  
	}
}
public String findCommarea(String commareaName, int aSize) throws CobolException
{
	Stack s = new Stack();
	Stack groupStack = new Stack();
	StringTokenizer st = new StringTokenizer(commareaName, ".");

	while ( st.hasMoreTokens() )
		s.push(st.nextToken());
	while ( !s.empty() )
	{
		groupStack.push(s.peek());
		s.pop();
	}
	int size = groupStack.size();

	if ( ((String)groupStack.peek()).equalsIgnoreCase("DFHCOMMAREA") && size > 1 )
	{
		Object[] obj = {commareaName};
		throw new CobolException(CobolResource.getString("_EXC_Cannot_import_record", obj));
	}

	while ( ready() )
 	{
		String record = readln();
		if ( record != null )
	 	{
			try {
				
				st = new StringTokenizer(record, " ");
		  		int levelNumber = Integer.parseInt(st.nextToken());
		   		String recordName = st.nextToken();
		   		String groupName = groupStack.peek().toString();
		   		if ( recordName.equalsIgnoreCase(groupName) )
		   		{
			  		if ( size == 1 )
			  		{
				 		if ( levelNumber == 1 )
							groupStack.pop();
			  		}
			  		else
				 		groupStack.pop();
		   		}
		   		if ( groupStack.empty() )
			  		return record;
			  		
			} catch ( NumberFormatException exc ) {}
			// Puts an empty line in the tmp file for lines above the commarea
			writeln();
	  	}
  	}

 	closeFile();

 	if ( aSize > 1 && commareaName.equalsIgnoreCase("DFHCOMMAREA") )
 	{
		throw new CobolException(CobolResource.getString("_EXC_DFHCOMMAREA_could_not_be_found"));
 	}
 	else
 	{
		Object[] obj = {commareaName};
		throw new CobolException(CobolResource.getString("_EXC_Commarea_could_not_be_found", obj));
 	}
}
/**
 * Insert the method's description here.
 * Creation date: (4/21/99 12:01:04 PM)
 * @param replaceList java.util.Vector
 * @param replaceByList java.util.Vector
 */
public boolean generateToString() throws CobolException
{
	try {
		while ( ready() )
		{
			String record = readln();
			if ( record != null )
			{
				StringTokenizer st = new StringTokenizer(record);
				String token = st.nextToken();
				try {
				 	int levelNumber = Integer.parseInt(token);
				  	if ( levelNumber > commareaLevel )
				  	{
						if ( levelNumber != 66 && levelNumber != 77 )
					   		writeln(record);
						else
						 	writeln();
				   	}
				   	else
					  	return true;
			   	} catch ( NumberFormatException exc ) {
				   	if ( token.equalsIgnoreCase("COPY") )
				   		writeln(record);
				   	else {
					   	if(token.equalsIgnoreCase("EJECT") || token.equalsIgnoreCase("*CONTROL") || token.equalsIgnoreCase("*CBL") || token.equalsIgnoreCase("SKIP1") || token.equalsIgnoreCase("SKIP2") || token.equalsIgnoreCase("SKIP3") || token.equalsIgnoreCase("TITLE"))
					   		writeln();
					   	else
							return false;
				   	}
			 	}
			}
		}

		return false;
		
	} finally {
		closeFile();
		try {
			close();
		}
		catch(Exception ignored) {}
	}
}
public String generateToString(String commareaName, int aSize) throws CobolException
{
	String recordName = new String();
	String commarea = findCommarea(commareaName, aSize);

	boolean isElementaryItem = false;
	StringTokenizer st = new StringTokenizer(commarea, " ");
	commareaLevel = Integer.parseInt(st.nextToken());
	recordName = st.nextToken();
	StringBuffer recordBuffer = new StringBuffer("02 "+recordName);

	while ( st.hasMoreTokens() )
	{
		String token = st.nextToken();
		if (token.equalsIgnoreCase("PIC") || token.equalsIgnoreCase("PICTURE") || token.equalsIgnoreCase("COMP-1") || token.equalsIgnoreCase("COMPUTATIONAL-1") || token.equalsIgnoreCase("COMP-2") || token.equalsIgnoreCase("COMPUTATIONAL-2" ) )
			isElementaryItem = true;

		recordBuffer.append(" "+token);
	}

	if ( isElementaryItem )
	{
		writeln("01 "+recordName+"- .");
		writeln(recordBuffer.toString());
	}	 
	else
	{
		writeln(commarea);
		generateToString();	
	}

	return recordName;
}
public String readln() throws CobolException
{
	String line = new String();
	StringBuffer recordBuffer = new StringBuffer();
	int counter = -1;

	while ( !recordBuffer.toString().endsWith(".") )
	{
		try {
			
			line = f_in.readLine();
			
		} catch (IOException exc) {
			Object[] obj = {inFile};
			throw new CobolException(CobolResource.getString("_EXC_Error_reading_file", obj));			  
		}
	
		if ( line == null )
		{
			if ( recordBuffer.toString().trim().length() > 0 )
				return recordBuffer.toString();
			else
				return null;
		}
		if ( line.length() > 6 && line.charAt(6) != '*' && line.charAt(6) != '/' )
		{
			counter++;
			if ( line.length() > 72 )
		 		recordBuffer.append(" "+line.substring(7, 72).trim());
			else
				recordBuffer.append(" "+line.substring(7).trim());
		}
		else
			writeln();
	}

	for (int i=0; i<counter; i++)
		writeln();

	String recordStr = recordBuffer.toString();
	int index = recordStr.indexOf(" COPY ");
	if ( index != -1 && index > 0 )
	{
		recordBuffer = new StringBuffer();
		java.util.StringTokenizer st = new java.util.StringTokenizer(recordStr);
		while ( st.hasMoreTokens() )
		{
			String str = st.nextToken();
			recordBuffer.append(str);
			if ( str.toString().endsWith(".") )
				recordBuffer.insert(recordBuffer.length() - 1, ' ');
			recordBuffer.append(' ');
		}
	}
	else
		recordBuffer.insert(recordBuffer.length() - 1, ' ');

	return recordBuffer.toString();
}
public boolean ready() throws CobolException
{
	try {

		return f_in.ready();

	} catch (IOException exc) {
		Object[] obj = {inFile};
		throw new CobolException(CobolResource.getString("_EXC_Error_reading_file", obj));			  
	}
}
/**
 * This method was created in VisualAge.
 * @return java.lang.String
 * @param str java.lang.String
 */
public String replace(String recordStr, Vector replaceList, Vector replaceByList) throws CobolException
{
	StringBuffer recordBuffer = new StringBuffer();
	StreamTokenizer f = new StreamTokenizer(new StringReader(recordStr));
	f.ordinaryChars(32, 126);
	f.wordChars(32, 126);
	f.whitespaceChars(' ',' ');
	f.quoteChar('\'');
	f.quoteChar('"');
	f.eolIsSignificant(true);
	Vector tokenList = new Vector();
	boolean match = false;
	boolean eof = false;

	while ( true )
	{
		for (int i=0;i<replaceList.size();i++)
		{
			Vector operand1 = (Vector)replaceList.elementAt(i);
			String operand1Str = operand1.firstElement().toString();
			for (int k=1;k<operand1.size();k++)
				operand1Str = operand1Str + " " + operand1.elementAt(k);
			if ( !eof )
			{
				for (int k=0;k<operand1.size()-tokenList.size();k++)
				{
					int ttype;
					try {
						ttype = f.nextToken();
					} catch (IOException exc) {
						throw new CobolException(CobolResource.getString("_EXC_Import_failed"));
					}
					if ( ttype == StreamTokenizer.TT_EOF )
					{
						eof = true;
						break;
					}
					else if ( ttype == StreamTokenizer.TT_EOL )
						tokenList.addElement(new Character((char)ttype).toString());
					else if ( ttype == StreamTokenizer.TT_WORD )
						tokenList.addElement(f.sval);
					else if ( ttype == '\'' || ttype == '"' )
					{
						String character = new Character((char)ttype).toString();
						tokenList.addElement(character+f.sval+character);
					}
				}
			}
			if ( eof && tokenList.isEmpty() )
				return recordBuffer.toString().trim();
			int size = operand1.size();
			if ( tokenList.size() < size )
				size = tokenList.size();
			String tokenStr = tokenList.firstElement().toString();
			for (int k=1;k<size;k++)
				tokenStr = tokenStr + " " + tokenList.elementAt(k);
			if ( operand1Str.startsWith(":") && operand1Str.endsWith(":") || ( operand1Str.startsWith("(") && operand1Str.endsWith(")") ) )
			{
				int index = tokenStr.toLowerCase().indexOf(operand1Str.toLowerCase());
				if ( index != -1 )
				{
					Vector operand2 = (Vector)replaceByList.elementAt(i);
					String operand2Str = operand2.firstElement().toString();
					for (int k=1;k<operand2.size();k++)
						operand2Str = operand2Str + " " + operand2.elementAt(k);
					recordBuffer.append(tokenStr.substring(0, index)+operand2Str+tokenStr.substring(index+operand1Str.length()));
					match = true;
					for (int k=0;k<operand1.size();k++)
						tokenList.removeElementAt(0);
					break;
				}
			}
			else if ( operand1Str.startsWith(":") )
			{
				if ( tokenStr.toLowerCase().endsWith(operand1Str.toLowerCase()) )
				{
					Vector operand2 = (Vector)replaceByList.elementAt(i);
					String operand2Str = operand2.firstElement().toString();
					for (int k=1;k<operand2.size();k++)
						operand2Str = operand2Str + " " + operand2.elementAt(k);
					int index = tokenStr.toLowerCase().indexOf(operand1Str.toLowerCase());
					recordBuffer.append(tokenStr.substring(0, index)+operand2Str);
					match = true;
					for (int k=0;k<operand1.size();k++)
						tokenList.removeElementAt(0);
					break;
				}
			}
			else if ( operand1Str.endsWith(":") )
			{
				if ( tokenStr.toLowerCase().startsWith(operand1Str.toLowerCase()) )
				{
					Vector operand2 = (Vector)replaceByList.elementAt(i);
					String operand2Str = operand2.firstElement().toString();
					for (int k=1;k<operand2.size();k++)
						operand2Str = operand2Str + " " + operand2.elementAt(k);
					recordBuffer.append(operand2Str+tokenStr.substring(operand1Str.length()));
					match = true;
					for (int k=0;k<operand1.size();k++)
						tokenList.removeElementAt(0);
					break;
				}
			}
			else
			{
				if ( tokenStr.equalsIgnoreCase(operand1Str) )
				{
					Vector operand2 = (Vector)replaceByList.elementAt(i);
					String operand2Str = operand2.firstElement().toString();
					for (int k=1;k<operand2.size();k++)
						operand2Str = operand2Str + " " + operand2.elementAt(k);
					recordBuffer.append(operand2Str);
					match = true;
					for (int k=0;k<operand1.size();k++)
						tokenList.removeElementAt(0);
					break;
				}
			}
		}
		if ( !match )
		{
			recordBuffer.append(tokenList.firstElement());
			tokenList.removeElementAt(0);
		}
		else
			match = false;
		recordBuffer.append(" ");
	}
	
}
public void writeln()
{
	write(System.getProperty("line.separator"));
}
public void writeln(String s)
{
	write(s);
	writeln();
}
}
